@@ -4,7 +4,7 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from ctypes import WinDLL, POINTER, WINFUNCTYPE, GetLastError, Structure, c_ulong, c_ushort, c_ubyte, c_int, c_long, c_void_p, c_size_t, c_char, byref
+from ctypes import WinDLL, POINTER, WINFUNCTYPE, GetLastError, Structure, c_ulong, c_ushort, c_ubyte, c_int, c_long, c_void_p, c_size_t, c_char, byref, sizeof
 from ctypes.wintypes import HWND, DWORD, WPARAM, LPARAM, HDC, HMONITOR, HMODULE, SHORT, ATOM, RECT, POINT, MAX_PATH, WCHAR, BYTE
 from ctypes.wintypes import HANDLE, LPCWSTR, UINT, INT, BOOL, WORD, HGDIOBJ, LONG, LPVOID, HBITMAP, LPCSTR, LPWSTR, HWINSTA, HINSTANCE, HMENU
 #imported from this module but not used here:
@@ -118,6 +118,32 @@ class BITMAPV5HEADER(Structure):
         ('bV5Reserved',         DWORD),
     ]
 
+CCHDEVICENAME = 32
+class MONITORINFOEX(Structure):
+    _fields_ = [
+        ('cbSize', DWORD),
+        ('rcMonitor', RECT),
+        ('rcWork', RECT),
+        ('dwFlags', DWORD),
+        ('szDevice', WCHAR * CCHDEVICENAME),
+        ]
+
+def GetMonitorInfo(hmonitor):
+    info = MONITORINFOEX()
+    info.szDevice = ""
+    info.cbSize = sizeof(MONITORINFOEX)
+    if not GetMonitorInfoW(hmonitor, byref(info)):
+        raise WindowsError()
+    monitor = info.rcMonitor.left, info.rcMonitor.top, info.rcMonitor.right, info.rcMonitor.bottom
+    work = info.rcWork.left, info.rcWork.top, info.rcWork.right, info.rcWork.bottom
+    return  {
+        "Work"      : work,
+        "Monitor"   : monitor,
+        "Flags"     : info.dwFlags,
+        "Device"    : info.szDevice or "",
+        }
+
+	
 kernel32 = WinDLL("kernel32", use_last_error=True)
 SetConsoleTitleA = kernel32.SetConsoleTitleA
 GetConsoleScreenBufferInfo = kernel32.GetConsoleScreenBufferInfo
@@ -189,6 +215,7 @@ EnumDisplayMonitors = user32.EnumDisplayMonitors
 MonitorFromWindow = user32.MonitorFromWindow
 MonitorFromWindow.restype = HMONITOR
 GetMonitorInfoW = user32.GetMonitorInfoW
+GetMonitorInfoW.restype = BOOL
 UnhookWindowsHookEx = user32.UnhookWindowsHookEx
 CallNextHookEx = user32.CallNextHookEx
 SetWindowsHookExA = user32.SetWindowsHookExA
@@ -362,6 +389,8 @@ GetObjectA.restype = INT
 
 #wrap EnumDisplayMonitors to hide the callback function:
 MonitorEnumProc = WINFUNCTYPE(BOOL, HMONITOR, HDC, POINTER(RECT), LPARAM)
+EnumDisplayMonitors.argtypes = [HDC, POINTER(RECT), MonitorEnumProc, LPARAM]
+EnumDisplayMonitors.restype = BOOL
 _EnumDisplayMonitors = EnumDisplayMonitors
 def EnumDisplayMonitors():
     results = []
@@ -369,7 +398,7 @@ def EnumDisplayMonitors():
         results.append(monitor)
         return 1
     callback = MonitorEnumProc(_callback)
-    _EnumDisplayMonitors(0, 0, callback, 0)
+    _EnumDisplayMonitors(0, None, callback, 0)
     return results
 
 def GetIntSystemParametersInfo(key):
