@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -8,20 +8,22 @@
 # Based on code from winswitch, itself based on "win32gui_taskbar demo"
 
 import ctypes
-from ctypes import POINTER, Structure, byref, WinDLL, c_void_p, sizeof
-from ctypes.wintypes import HWND, UINT, POINT, HICON, BOOL, HBITMAP, WCHAR, LONG, WORD, HANDLE, INT, DWORD
+from ctypes import POINTER, Structure, byref, WinDLL, c_void_p, sizeof, create_string_buffer
+from ctypes.wintypes import HWND, UINT, POINT, HICON, BOOL, WCHAR, DWORD
 
-from xpra.util import csv, XPRA_APP_ID
-from xpra.os_util import memoryview_to_bytes
+from xpra.util import csv, nonl, XPRA_GUID1, XPRA_GUID2, XPRA_GUID3, XPRA_GUID4
+from xpra.os_util import memoryview_to_bytes, bytestostr
 from xpra.platform.win32 import constants as win32con
 from xpra.platform.win32.common import (GUID, WNDCLASSEX, WNDPROC,
                                         GetSystemMetrics,
                                         GetCursorPos,
                                         PostMessageA,
                                         CreateWindowExA,
+                                        CreatePopupMenu, AppendMenu,
                                         LoadIconA,
-                                        DefWindowProcW, RegisterWindowMessageA,
-                                        RegisterClassExW,
+                                        DefWindowProcA, RegisterWindowMessageA,
+                                        RegisterClassExA,
+                                        ICONINFO, BITMAPV5HEADER,
                                         LoadImageW, CreateIconIndirect,
                                         GetDC, ReleaseDC,
                                         CreateBitmap, CreateDIBSection,
@@ -37,13 +39,9 @@ sprintf = ctypes.cdll.msvcrt.sprintf
 
 
 def GetProductInfo(dwOSMajorVersion=5, dwOSMinorVersion=0, dwSpMajorVersion=0, dwSpMinorVersion=0):
-    from xpra.platform.win32.common import kernel32
-    GetProductInfo = kernel32.GetProductInfo
-    PDWORD = POINTER(DWORD)
-    GetProductInfo.argtypes = [DWORD, DWORD, DWORD, DWORD, PDWORD]
-    GetProductInfo.restype  = BOOL
     product_type = DWORD(0)
-    v = GetProductInfo(dwOSMajorVersion, dwOSMinorVersion, dwSpMajorVersion, dwSpMinorVersion, byref(product_type))
+    from xpra.platform.win32.common import GetProductInfo as k32GetProductInfo
+    v = k32GetProductInfo(dwOSMajorVersion, dwOSMinorVersion, dwSpMajorVersion, dwSpMinorVersion, byref(product_type))
     log("GetProductInfo(%i, %i, %i, %i)=%i product_type=%s", dwOSMajorVersion, dwOSMinorVersion, dwSpMajorVersion, dwSpMinorVersion, v, product_type)
     return bool(v)
 #win7 is actually 6.1:
@@ -54,30 +52,20 @@ except AttributeError as e:
     log("cannot query GetProductInfo", exc_info=True)
     raise ImportError("cannot query GetProductInfo: %s" % e)
 
-class ICONINFO(Structure):
-    _fields_ = [
-        ('fIcon',       BOOL),
-        ('xHotspot',    DWORD),
-        ('yHotspot',    DWORD),
-        ('hbmMask',     HBITMAP),
-        ('hbmColor',    HBITMAP),
-    ]
-CreateIconIndirect.restype = HICON
-CreateIconIndirect.argtypes = [POINTER(ICONINFO)]
-
 if ISWIN7ORHIGHER:
     MAX_TIP_SIZE = 128
 else:
     MAX_TIP_SIZE = 64
 
-NOTIFYICONDATA_fields= [
+class NOTIFYICONDATA(Structure):
+    _fields_ = [
         ("cbSize",              DWORD),
         ("hWnd",                HWND),
         ("uID",                 UINT),
         ("uFlags",              UINT),
         ("uCallbackMessage",    UINT),
         ("hIcon",               HICON),
-        ("szTip",               WCHAR * 128),
+        ("szTip",               WCHAR * 64),
         ("dwState",             DWORD),
         ("dwStateMask",         DWORD),
         ("szInfo",              WCHAR * 256),
@@ -88,73 +76,19 @@ NOTIFYICONDATA_fields= [
         ("hBalloonIcon",        HICON),
     ]
 
-if ISWIN7ORHIGHER:
-    #full:
-    class NOTIFYICONDATA(Structure):
-        _fields_ = NOTIFYICONDATA_fields[:]
-else:
-    #winxp: V2
-    class NOTIFYICONDATA(Structure):
-        _fields_ = NOTIFYICONDATA_fields[:-2]
-
 shell32 = WinDLL("shell32", use_last_error=True)
-Shell_NotifyIcon = shell32.Shell_NotifyIcon
-Shell_NotifyIcon.restype = BOOL
-Shell_NotifyIcon.argtypes = [DWORD, POINTER(NOTIFYICONDATA)]
+Shell_NotifyIconW = shell32.Shell_NotifyIconW
+Shell_NotifyIconW.restype = BOOL
+Shell_NotifyIconW.argtypes = [DWORD, POINTER(NOTIFYICONDATA)]
 
 BI_RGB = 0
 BI_BITFIELDS = 0x00000003
-class CIEXYZ(Structure):
-    _fields_ = [
-        ('ciexyzX', DWORD),
-        ('ciexyzY', DWORD),
-        ('ciexyzZ', DWORD),
-    ]
-class CIEXYZTRIPLE(Structure):
-    _fields_ = [
-        ('ciexyzRed',   CIEXYZ),
-        ('ciexyzBlue',  CIEXYZ),
-        ('ciexyzGreen', CIEXYZ),
-    ]
-class BITMAPV5HEADER(Structure):
-    _fields_ = [
-        ('bV5Size',             DWORD),
-        ('bV5Width',            LONG),
-        ('bV5Height',           LONG),
-        ('bV5Planes',           WORD),
-        ('bV5BitCount',         WORD),
-        ('bV5Compression',      DWORD),
-        ('bV5SizeImage',        DWORD),
-        ('bV5XPelsPerMeter',    LONG),
-        ('bV5YPelsPerMeter',    LONG),
-        ('bV5ClrUsed',          DWORD),
-        ('bV5ClrImportant',     DWORD),
-        ('bV5RedMask',          DWORD),
-        ('bV5GreenMask',        DWORD),
-        ('bV5BlueMask',         DWORD),
-        ('bV5AlphaMask',        DWORD),
-        ('bV5CSType',           DWORD),
-        ('bV5Endpoints',        CIEXYZTRIPLE),
-        ('bV5GammaRed',         DWORD),
-        ('bV5GammaGreen',       DWORD),
-        ('bV5GammaBlue',        DWORD),
-        ('bV5Intent',           DWORD),
-        ('bV5ProfileData',      DWORD),
-        ('bV5ProfileSize',      DWORD),
-        ('bV5Reserved',         DWORD),
-    ]
-
-CreateDIBSection.restype = HBITMAP
-CreateDIBSection.argtypes = [HANDLE, POINTER(BITMAPV5HEADER), UINT, POINTER(c_void_p), HANDLE, DWORD]
-
-CreateBitmap.restype = HBITMAP
-CreateBitmap.argtypes = [INT, INT, UINT, UINT, POINTER(c_void_p)]
 
 XPRA_GUID = GUID()
-XPRA_GUID.Data1 = 0x67b3efa2
-XPRA_GUID.Data2 = 0xe470
-XPRA_GUID.Data3 = 0x4a5f
-XPRA_GUID.Data4 = (0xb6, 0x53, 0x6f, 0x6f, 0x98, 0xfe, 0x60, 0x81)
+XPRA_GUID.Data1 = XPRA_GUID1
+XPRA_GUID.Data2 = XPRA_GUID2
+XPRA_GUID.Data3 = XPRA_GUID3
+XPRA_GUID.Data4 = XPRA_GUID4
 
 FALLBACK_ICON = LoadIconA(0, win32con.IDI_APPLICATION)
 
@@ -207,6 +141,38 @@ BUTTON_MAP = {
             }
 
 
+def image_to_ICONINFO(img):
+    w, h = img.size
+    from xpra.codecs.argb.argb import rgba_to_bgra       #@UnresolvedImport
+    bgra = memoryview_to_bytes(rgba_to_bgra(img.tobytes("raw", "BGRA")))
+    return make_ICONINFO(w, h, bgra)
+
+def make_ICONINFO(w, h, bgra):
+    bitmap = 0
+    mask = 0
+    try:
+        bitmap = rgba_to_bitmap(bgra, w, h)
+        mask = CreateBitmap(w, h, 1, 1, None)
+
+        iconinfo = ICONINFO()
+        iconinfo.fIcon = True
+        iconinfo.hbmMask = mask
+        iconinfo.hbmColor = bitmap
+        hicon = CreateIconIndirect(byref(iconinfo))
+        log("CreateIconIndirect()=%#x", hicon)
+        if not hicon:
+            raise ctypes.WinError(ctypes.get_last_error())
+        return hicon
+    except Exception:
+        log.error("Error: failed to set tray icon", exc_info=True)
+        return FALLBACK_ICON
+    finally:
+        if mask:
+            DeleteObject(mask)
+        if bitmap:
+            DeleteObject(bitmap)
+
+
 def rgba_to_bitmap(rgba, w, h):
     header = BITMAPV5HEADER()
     header.bV5Size = sizeof(BITMAPV5HEADER)
@@ -229,7 +195,7 @@ def rgba_to_bitmap(rgba, w, h):
         ReleaseDC(None, hdc)
     assert dataptr and bitmap, "failed to create DIB section"
     log("CreateDIBSection(..) got bitmap=%#x, dataptr=%s", int(bitmap), dataptr)
-    img_data = ctypes.create_string_buffer(rgba)
+    img_data = create_string_buffer(rgba)
     ctypes.memmove(dataptr, byref(img_data), w*4*h)
     return bitmap
 
@@ -240,7 +206,7 @@ class win32NotifyIcon(object):
     #this allows us to know which hwnd refers to which instance:
     instances = {}
 
-    def __init__(self, app_id, title, move_callbacks, click_callback, exit_callback, command_callback=None, iconPathName=None):
+    def __init__(self, app_id=0, title="", move_callbacks=None, click_callback=None, exit_callback=None, command_callback=None, iconPathName=None):
         log("win32NotifyIcon: app_id=%i, title='%s'", app_id, title)
         self.app_id = app_id
         self.title = title
@@ -262,19 +228,25 @@ class win32NotifyIcon(object):
         self.reset_function = None
 
     def create_tray_window(self):
+        self.create_window()
+        self.register_tray()
+
+    def create_window(self):
         style = win32con.WS_OVERLAPPED | win32con.WS_SYSMENU
-        window_name = (self.title+" StatusIcon Window").decode()
+        window_name = u"%s StatusIcon Window" % bytestostr(self.title)
         self.hwnd = CreateWindowExA(0, NIclassAtom, window_name, style,
-            0, 0, win32con.CW_USEDEFAULT, win32con.CW_USEDEFAULT, \
+            win32con.CW_USEDEFAULT, win32con.CW_USEDEFAULT, 0, 0, \
             0, 0, NIwc.hInstance, None)
         if self.hwnd==0:
             raise ctypes.WinError(ctypes.get_last_error())
         log("hwnd=%#x", self.hwnd)
         UpdateWindow(self.hwnd)
-        r = Shell_NotifyIcon(NIM_ADD, self.make_nid(NIF_ICON | NIF_MESSAGE | NIF_TIP))
-        log("Shell_NotifyIcon ADD=%i", r)
+
+    def register_tray(self):
+        r = Shell_NotifyIconW(NIM_ADD, self.make_nid(NIF_ICON | NIF_MESSAGE | NIF_TIP))
+        log("Shell_NotifyIconW ADD=%i", r)
         if not r:
-            raise Exception("Shell_NotifyIcon failed to ADD")
+            raise Exception("Shell_NotifyIconW failed to ADD")
 
     def make_nid(self, flags):
         nid = NOTIFYICONDATA()
@@ -283,28 +255,23 @@ class win32NotifyIcon(object):
         nid.uCallbackMessage = win32con.WM_MENUCOMMAND
         nid.hIcon = self.current_icon
         #don't ask why we have to use sprintf to get what we want:
-        title = self.title[:MAX_TIP_SIZE-1]
-        sprintf(byref(nid,NOTIFYICONDATA.szTip.offset), title)
+        title = bytestostr(self.title[:MAX_TIP_SIZE-1])
+        nid.szTip = title
         nid.dwState = 0
         nid.dwStateMask = 0
+        nid.guidItem = XPRA_GUID
+        nid.uID = self.app_id
+        flags |= NIF_GUID
         #balloon notification bits:
         #szInfo
         #uTimeout
         #szInfoTitle
         #dwInfoFlags
         #hBalloonIcon
-        if ISWIN7ORHIGHER:
-            #flags |= NIF_SHOWTIP
-            if self.app_id==XPRA_APP_ID:
-                nid.guidItem = XPRA_GUID
-                flags |= NIF_GUID
-            else:
-                nid.uID = self.app_id
-            nid.uVersion = 4
-        else:
-            nid.uVersion = 3
+        #flags |= NIF_SHOWTIP
+        nid.uVersion = 4
         nid.uFlags = flags
-        log("make_nid(..)=%s tooltip='%s', app_id=%i, actual flags=%s", nid, title, self.app_id, csv([v for k,v in NIF_FLAGS.items() if k&flags]))
+        log("make_nid(..)=%s tooltip='%s', app_id=%i, actual flags=%s", nid, nonl(title), self.app_id, csv([v for k,v in NIF_FLAGS.items() if k&flags]))
         return nid
 
     def delete_tray_window(self):
@@ -312,8 +279,8 @@ class win32NotifyIcon(object):
             return
         try:
             nid = self.make_nid(0)
-            log("delete_tray_window(..) calling Shell_NotifyIcon(NIM_DELETE, %s)", nid)
-            Shell_NotifyIcon(NIM_DELETE, nid)
+            log("delete_tray_window(..) calling Shell_NotifyIconW(NIM_DELETE, %s)", nid)
+            Shell_NotifyIconW(NIM_DELETE, nid)
         except Exception as e:
             log.error("Error: failed to delete tray window")
             log.error(" %s", e)
@@ -326,19 +293,19 @@ class win32NotifyIcon(object):
 
     def set_tooltip(self, tooltip):
         self.title = tooltip
-        Shell_NotifyIcon(NIM_MODIFY, self.make_nid(NIF_ICON | NIF_MESSAGE | NIF_TIP))
+        Shell_NotifyIconW(NIM_MODIFY, self.make_nid(NIF_ICON | NIF_MESSAGE | NIF_TIP))
 
 
     def set_icon(self, iconPathName):
         hicon = self.LoadImage(iconPathName)
         self.do_set_icon(hicon)
-        Shell_NotifyIcon(NIM_MODIFY, self.make_nid(NIF_ICON))
+        Shell_NotifyIconW(NIM_MODIFY, self.make_nid(NIF_ICON))
         self.reset_function = (self.set_icon, iconPathName)
 
     def do_set_icon(self, hicon):
-        log("do_set_icon(%s)", hicon)
+        log("do_set_icon(%#x)", hicon)
         self.current_icon = hicon
-        Shell_NotifyIcon(NIM_MODIFY, self.make_nid(NIF_ICON))
+        Shell_NotifyIconW(NIM_MODIFY, self.make_nid(NIF_ICON))
 
     def set_icon_from_data(self, pixels, has_alpha, w, h, rowstride, options={}):
         #this is convoluted but it works..
@@ -356,37 +323,24 @@ class win32NotifyIcon(object):
         icon_h = GetSystemMetrics(win32con.SM_CYSMICON)
         if w!=icon_w or h!=icon_h:
             log("resizing tray icon to %ix%i", icon_w, icon_h)
-            img = img.resize((w, h), Image.ANTIALIAS)
+            img = img.resize((icon_w, icon_h), Image.ANTIALIAS)
+            rowstride = w*4
 
-        bitmap = 0
-        mask = 0
-        try:
-            from xpra.codecs.argb.argb import rgba_to_bgra       #@UnresolvedImport
-            bgra = memoryview_to_bytes(rgba_to_bgra(img.tobytes()))
-            bitmap = rgba_to_bitmap(bgra, icon_w, icon_h)
-            mask = CreateBitmap(icon_w, icon_h, 1, 1, None)
-
-            iconinfo = ICONINFO()
-            iconinfo.fIcon = True
-            iconinfo.hbmMask = mask
-            iconinfo.hbmColor = bitmap
-            hicon = CreateIconIndirect(byref(iconinfo))
-            log("CreateIconIndirect()=%s", hicon)
-            if not hicon:
-                raise ctypes.WinError(ctypes.get_last_error())
-        except Exception:
-            log.error("Error: failed to set tray icon", exc_info=True)
-            hicon = FALLBACK_ICON
-        finally:
-            if mask:
-                DeleteObject(mask)
-            if bitmap:
-                DeleteObject(bitmap)
+        hicon = image_to_ICONINFO(img)
         self.do_set_icon(hicon)
         UpdateWindow(self.hwnd)
         self.reset_function = (self.set_icon_from_data, pixels, has_alpha, w, h, rowstride)
 
-    def LoadImage(self, iconPathName, fallback=FALLBACK_ICON):
+    def LoadImage(self, iconPathName):
+        if not iconPathName:
+            return None
+        image = self.image_cache.get(iconPathName)
+        if not image:
+            image = self.doLoadImage(iconPathName)
+            self.image_cache[iconPathName] = image
+        return image
+
+    def doLoadImage(self, iconPathName):
         v = fallback
         if iconPathName:
             icon_flags = win32con.LR_LOADFROMFILE | win32con.LR_DEFAULTSIZE
@@ -422,7 +376,7 @@ class win32NotifyIcon(object):
             log.error(" %s", e)
 
     def OnCommand(self, hwnd, msg, wparam, lparam):
-        cb = getattr(self, "command_callback", None)
+        cb = self.command_callback
         log("OnCommand%s callback=%s", (hwnd, msg, wparam, lparam), cb)
         if cb:
             cid = wparam & 0xFFFF
@@ -482,39 +436,27 @@ def NotifyIconWndProc(hwnd, msg, wParam, lParam):
     #log("potential matching win32 constants for message: %s", [x for x in dir(win32con) if getattr(win32con, x)==msg])
     if instance and fn:
         return fn(instance, hwnd, msg, wParam, lParam) or 0
-    return DefWindowProcW(hwnd, msg, wParam, lParam)
+    return DefWindowProcA(hwnd, msg, wParam, lParam)
 
 NIwc = WNDCLASSEX()
 NIwc.cbSize = sizeof(WNDCLASSEX)
 NIwc.style = win32con.CS_HREDRAW | win32con.CS_VREDRAW
 NIwc.lpfnWndProc = WNDPROC(NotifyIconWndProc)
-NIwc.cbClsExtra = 0
-NIwc.cbWndExtra = 0
 NIwc.hInstance = GetModuleHandleA(0)
-NIwc.hIcon = 0
-NIwc.hCursor = 0
 NIwc.hBrush = GetStockObject(win32con.WHITE_BRUSH)
-NIwc.lpszMenuName = 0
 NIwc.lpszClassName = u"win32NotifyIcon"
-NIwc.hIconSm = 0
 
-NIclassAtom = RegisterClassExW(byref(NIwc))
+NIclassAtom = RegisterClassExA(byref(NIwc))
 if NIclassAtom==0:
     raise ctypes.WinError(ctypes.get_last_error())
-log("RegisterClassExW(%s)=%i", NIwc.lpszClassName, NIclassAtom)
+log("RegisterClassExA(%s)=%i", NIwc.lpszClassName, NIclassAtom)
+
 
 def main():
     import os
-    import sys
     from xpra.platform.win32.common import user32
 
     def click_callback(button, pressed):
-        CreatePopupMenu = user32.CreatePopupMenu
-        CreatePopupMenu.restype = ctypes.wintypes.HMENU
-        CreatePopupMenu.argtypes = []
-        AppendMenu = user32.AppendMenuW
-        AppendMenu.restype = ctypes.wintypes.BOOL
-        AppendMenu.argtypes = [ctypes.wintypes.HMENU, ctypes.wintypes.UINT, ctypes.wintypes.UINT, ctypes.wintypes.LPCWSTR]
         menu = CreatePopupMenu()
         AppendMenu(menu, win32con.MF_STRING, 1024, u"Generate balloon")
         AppendMenu(menu, win32con.MF_STRING, 1025, u"Exit")
@@ -528,7 +470,21 @@ def main():
     def command_callback(hwnd, cid):
         if cid == 1024:
             from xpra.platform.win32.win32_balloon import notify
-            notify(hwnd, "hello", "world")
+            from xpra.os_util import BytesIOClass
+            try:
+                from PIL import Image   #@UnresolvedImport
+                img = Image.open("icons\\printer.png")
+                buf = BytesIOClass()
+                img.save(buf, "PNG")
+                data = buf.getvalue()
+                buf.close()
+                icon = (b"png", img.size[0], img.size[1], data)
+            except Exception as e:
+                print("could not find icon: %s" % (e,))
+                icon = None
+            else:
+                pass
+            notify(hwnd, 0, "hello", "world", timeout=1000, icon=icon)
         elif cid == 1025:
             print("Goodbye")
             DestroyWindow(hwnd)
@@ -538,8 +494,13 @@ def main():
     def win32_quit():
         PostQuitMessage(0) # Terminate the app.
 
-    iconPathName = os.path.abspath(os.path.join( sys.prefix, "pyc.ico"))
-    tray = win32NotifyIcon("test", move_callbacks=None, click_callback=click_callback, exit_callback=win32_quit, command_callback=command_callback, iconPathName=iconPathName)
+    from xpra.platform.paths import get_app_dir
+    idir = os.path.abspath(get_app_dir())
+    wdir = os.path.join(idir, "win32")
+    if os.path.exists(wdir):
+        idir = wdir
+    iconPathName = os.path.join(idir, "xpra.ico")
+    tray = win32NotifyIcon(0, "test", move_callbacks=None, click_callback=click_callback, exit_callback=win32_quit, command_callback=command_callback, iconPathName=iconPathName)
     #pump messages:
     msg = ctypes.wintypes.MSG()
     pMsg = ctypes.pointer(msg)
