@@ -21,7 +21,8 @@ class Keyboard(KeyboardBase):
             from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings   #@UnresolvedImport
             self.keyboard_bindings = X11KeyboardBindings()
         except Exception as e:
-            log.warn("failed load posix keyboard bindings: %s", e)
+            log.error("Error: failed to load posix keyboard bindings")
+            log.error(" %s", e)
             self.keyboard_bindings = None
 
 
@@ -35,36 +36,42 @@ class Keyboard(KeyboardBase):
             log.warn("keyboard bindings are not available, expect keyboard mapping problems")
             return {}, [], []
         try:
-            mod_mappings = self.keyboard_bindings.get_modifier_mappings()
-            if mod_mappings:
-                #ie: {"shift" : ["Shift_L", "Shift_R"], "mod1" : "Meta_L", ...]}
-                log("modifier mappings=%s", mod_mappings)
-                meanings = {}
-                for modifier,keys in mod_mappings.items():
-                    for _,keyname in keys:
-                        meanings[keyname] = modifier
-                #probably a GTK bug? but easier to put here
-                mod_missing = []
-                numlock_mod = meanings.get("Num_Lock", [])
-                if numlock_mod:
-                    mod_missing.append(numlock_mod)
-                return  meanings, [], mod_missing
-        except Exception as e:
-            log.error("failed to use native get_modifier_mappings: %s", e, exc_info=True)
+            from xpra.gtk_common.error import xsync
+            with xsync:
+                mod_mappings = self.keyboard_bindings.get_modifier_mappings()
+                if mod_mappings:
+                    #ie: {"shift" : ["Shift_L", "Shift_R"], "mod1" : "Meta_L", ...]}
+                    log("modifier mappings=%s", mod_mappings)
+                    meanings = {}
+                    for modifier,keys in mod_mappings.items():
+                        for _,keyname in keys:
+                            meanings[keyname] = modifier
+                    #probably a GTK bug? but easier to put here
+                    mod_missing = []
+                    numlock_mod = meanings.get("Num_Lock", [])
+                    if numlock_mod:
+                        mod_missing.append(numlock_mod)
+                    return  meanings, [], mod_missing
+        except Exception:
+            log.error("failed to use native get_modifier_mappings", exc_info=True)
+        return {}, [], []
 
     def get_x11_keymap(self):
         if not self.keyboard_bindings:
             return  {}
         try:
-            return self.keyboard_bindings.get_keycode_mappings()
-        except Exception as e:
-            log.error("failed to use raw x11 keymap: %s", e)
+            from xpra.gtk_common.error import xsync
+            with xsync:
+                return self.keyboard_bindings.get_keycode_mappings()
+        except Exception:
+            log.error("Error: failed to use raw x11 keymap", exc_info=True)
+        return  {}
 
 
     def get_keymap_spec(self):
         log("get_keymap_spec() keyboard_bindings=%s", self.keyboard_bindings)
         if not self.keyboard_bindings:
-            return None
+            return None, None, {}
         _query_struct = self.keyboard_bindings.getXkbProperties()
         _query = xkbmap_query_tostring(_query_struct)
         log("get_keymap_spec() Xkb query tostring(%s)=%s", _query_struct, _query)
@@ -92,9 +99,12 @@ class Keyboard(KeyboardBase):
     def get_layout_spec(self):
         layout = ""
         layouts = []
+        options = ""
         v = None
         if self.keyboard_bindings:
-            v = self.keyboard_bindings.getXkbProperties().get("layout")
+            props = self.keyboard_bindings.getXkbProperties()
+            v = props.get("layout")
+            options = props.get("options", "")
         if not v:
             #fallback:
             v = self.get_xkb_rules_names_property()
@@ -109,7 +119,7 @@ class Keyboard(KeyboardBase):
                 return v.encode("latin1")
             except:
                 return str(v)
-        return s(layout), [s(x) for x in layouts], "", None
+        return s(layout), [s(x) for x in layouts], "", None, options
 
 
     def get_keyboard_repeat(self):
