@@ -7,7 +7,7 @@
 import socket
 import os
 import signal
-from threading import Timer
+from threading import Timer, RLock
 from time import sleep
 
 from xpra.log import Logger
@@ -18,18 +18,19 @@ enclog = Logger("encoding")
 from xpra.server.server_core import get_server_info, get_thread_info
 from xpra.scripts.server import deadly_signal
 from xpra.net import compression
+from xpra.net.net_util import get_network_caps
 from xpra.net.compression import Compressed, compressed_wrapper
-from xpra.net.protocol import Protocol, get_network_caps
+from xpra.net.protocol import Protocol
 from xpra.codecs.loader import load_codecs, get_codec
 from xpra.codecs.image_wrapper import ImageWrapper
 from xpra.codecs.video_helper import getVideoHelper, PREFERRED_ENCODER_ORDER
-from xpra.os_util import Queue, SIGNAMES, strtobytes, memoryview_to_bytes, getuid, getgid, monotonic_time, get_username_for_uid, setuidgid
-from xpra.util import flatten_dict, typedict, updict, repr_ellipsized, xor, std, envint, envbool, csv, \
+from xpra.os_util import Queue, SIGNAMES, bytestostr, getuid, getgid, monotonic_time, get_username_for_uid, setuidgid, strtobytes
+from xpra.util import flatten_dict, typedict, updict, repr_ellipsized, xor, envint, envbool, csv, first_time, AtomicInteger, \
     LOGIN_TIMEOUT, CONTROL_COMMAND_ERROR, AUTHENTICATION_ERROR, CLIENT_EXIT_TIMEOUT, SERVER_SHUTDOWN
 from xpra.version_util import XPRA_VERSION
 from xpra.make_thread import start_thread
 from xpra.scripts.config import parse_number, parse_bool
-from xpra.scripts.main import full_version_str
+from xpra.version_util import full_version_str
 from xpra.server.socket_util import create_unix_domain_socket
 from xpra.platform.dotxpra import DotXpra
 from xpra.net.bytestreams import SocketConnection, SOCKET_TIMEOUT
@@ -47,6 +48,7 @@ PROXY_QUEUE_SIZE = envint("XPRA_PROXY_QUEUE_SIZE", 10)
 PASSTHROUGH = envbool("XPRA_PROXY_PASSTHROUGH", False)
 MAX_CONCURRENT_CONNECTIONS = 20
 VIDEO_TIMEOUT = 5                  #destroy video encoder after N seconds of idle state
+LEGACY_SALT_DIGEST = envbool("XPRA_LEGACY_SALT_DIGEST", True)
 
 
 def set_blocking(conn):
@@ -64,7 +66,7 @@ class ProxyInstanceProcess(Process):
 
     def __init__(self, uid, gid, env_options, session_options, socket_dir,
                  video_encoder_modules, csc_modules,
-                 client_conn, client_state, cipher, encryption_key, server_conn, caps, message_queue):
+                 client_conn, disp_desc, client_state, cipher, encryption_key, server_conn, caps, message_queue):
         Process.__init__(self, name=str(client_conn))
         self.uid = uid
         self.gid = gid
@@ -74,6 +76,7 @@ class ProxyInstanceProcess(Process):
         self.video_encoder_modules = video_encoder_modules
         self.csc_modules = csc_modules
         self.client_conn = client_conn
+        self.disp_desc = disp_desc
         self.client_state = client_state
         self.cipher = cipher
         self.encryption_key = encryption_key
@@ -81,7 +84,7 @@ class ProxyInstanceProcess(Process):
         self.caps = caps
         log("ProxyProcess%s", (uid, gid, env_options, session_options, socket_dir,
                                video_encoder_modules, csc_modules,
-                               client_conn, repr_ellipsized(str(client_state)), cipher, encryption_key, server_conn,
+                               client_conn, disp_desc, repr_ellipsized(str(client_state)), cipher, encryption_key, server_conn,
                                "%s: %s.." % (type(caps), repr_ellipsized(str(caps))), message_queue))
         self.client_protocol = None
         self.server_protocol = None
@@ -103,6 +106,9 @@ class ProxyInstanceProcess(Process):
         self.control_socket_path = None
         self.potential_protocols = []
         self.max_connections = MAX_CONCURRENT_CONNECTIONS
+        self.timer_id = AtomicInteger()
+        self.timers = {}
+        self.timer_lock = RLock()
 
     def server_message_queue(self):
         while True:
@@ -120,7 +126,7 @@ class ProxyInstanceProcess(Process):
             else:
                 log.error("unexpected proxy server message: %s", m)
 
-    def signal_quit(self, signum, frame):
+    def signal_quit(self, signum, frame=None):
         log.info("")
         log.info("proxy process pid %s got signal %s, exiting", os.getpid(), SIGNAMES.get(signum, signum))
         self.exit = True
@@ -128,30 +134,81 @@ class ProxyInstanceProcess(Process):
         signal.signal(signal.SIGTERM, deadly_signal)
         self.stop(SIGNAMES.get(signum, signum))
 
+
+    def source_remove(self, tid):
+        log("source_remove(%i)", tid)
+        with self.timer_lock:
+            try:
+                timer = self.timers[tid]
+                if timer is not None:
+                    del self.timers[tid]
+                if timer:
+                    timer.cancel()
+            except:
+                pass
+
     def idle_add(self, fn, *args, **kwargs):
-        #we emulate gobject's idle_add using a simple queue
-        self.main_queue.put((fn, args, kwargs))
+        tid = self.timer_id.increase()
+        self.main_queue.put((self.idle_repeat_call, (tid, fn, args, kwargs), {}))
+        #add an entry,
+        #but use the value False to stop us from trying to call cancel()
+        self.timers[tid] = False
+        return tid
+
+    def idle_repeat_call(self, tid, fn, args, kwargs):
+        if tid not in self.timers:
+            return False    #cancelled
+        return fn(*args, **kwargs)
 
     def timeout_add(self, timeout, fn, *args, **kwargs):
-        #emulate gobject's timeout_add using idle add and a Timer
-        #using custom functions to cancel() the timer when needed
-        def idle_exec():
-            v = fn(*args, **kwargs)
-            if bool(v):
-                self.timeout_add(timeout, fn, *args, **kwargs)
-            return False
-        def timer_exec():
-            #just run via idle_add:
-            self.idle_add(idle_exec)
-        Timer(timeout/1000.0, timer_exec).start()
+        tid = self.timer_id.increase()
+        self.do_timeout_add(tid, timeout, fn, *args, **kwargs)
+        return tid
+
+    def do_timeout_add(self, tid, timeout, fn, *args, **kwargs):
+        #emulate glib's timeout_add using Timers
+        args = (tid, timeout, fn, args, kwargs)
+        t = Timer(timeout/1000.0, self.queue_timeout_function, args)
+        self.timers[tid] = t
+        t.start()
+
+    def queue_timeout_function(self, tid, timeout, fn, fn_args, fn_kwargs):
+        if tid not in self.timers:
+            return      #cancelled
+        #add to run queue:
+        mqargs = [tid, timeout, fn, fn_args, fn_kwargs]
+        self.main_queue.put((self.timeout_repeat_call, mqargs, {}))
+
+    def timeout_repeat_call(self, tid, timeout, fn, fn_args, fn_kwargs, **kwargs):
+        #executes the function then re-schedules it (if it returns True)
+        if tid not in self.timers:
+            return      #cancelled
+        v = fn(*fn_args, **fn_kwargs)
+        if bool(v):
+            #create a new timer with the same tid:
+            with self.timer_lock:
+                if tid in self.timers:
+                    self.do_timeout_add(tid, timeout, fn, *fn_args, **fn_kwargs)
+        else:
+            try:
+                del self.timers[tid]
+            except:
+                pass
+        #we do the scheduling via timers, so always return False here
+        #so that the main queue won't re-schedule this function call itself:
+        return False
+
+
+    def setproctitle(self, title):
+        try:
+            import setproctitle
+            setproctitle.setproctitle(title)
+        except ImportError as e:
+            log("setproctitle not installed: %s", e)
 
     def run(self):
         log("ProxyProcess.run() pid=%s, uid=%s, gid=%s", os.getpid(), getuid(), getgid())
-        try:
-            import setproctitle
-            setproctitle.setproctitle("Xpra Proxy Instance for %s" % self.server_conn)
-        except ImportError as e:
-            log("setproctitle not installed: %s", e)
+        self.setproctitle("Xpra Proxy Instance for %s" % self.server_conn)
         setuidgid(self.uid, self.gid)
         if self.env_options:
             #TODO: whitelist env update?
@@ -181,6 +238,7 @@ class ProxyInstanceProcess(Process):
         self.client_protocol.restore_state(self.client_state)
         self.server_protocol = Protocol(self, self.server_conn, self.process_server_packet, self.get_server_packet)
         #server connection tweaks:
+        self.server_protocol.large_packets.append("input-devices")
         self.server_protocol.large_packets.append("draw")
         self.server_protocol.large_packets.append("window-icon")
         self.server_protocol.large_packets.append("keymap-changed")
@@ -248,11 +306,11 @@ class ProxyInstanceProcess(Process):
         enclog("encoder types found: %s", tuple(encoder_types))
         #remove duplicates and use preferred order:
         order = PREFERRED_ENCODER_ORDER[:]
-        for x in list(encoder_types):
+        for x in tuple(encoder_types):
             if x not in order:
                 order.append(x)
         self.video_encoder_types = [x for x in order if x in encoder_types]
-        enclog.info("proxy video encoders: %s", ", ".join(self.video_encoder_types or ["none",]))
+        enclog.info("proxy video encoders: %s", csv(self.video_encoder_types or ["none",]))
 
 
     def create_control_socket(self):
@@ -275,7 +333,7 @@ class ProxyInstanceProcess(Process):
             log.warn("Warning: failed to create socket directory '%s'", d)
             log.warn(" %s", e)
         try:
-            sock, self.control_socket_cleanup = create_unix_domain_socket(sockpath, None, 0o600)
+            sock, self.control_socket_cleanup = create_unix_domain_socket(sockpath, 0o600)
             sock.listen(5)
         except Exception as e:
             log("create_unix_domain_socket failed for '%s'", sockpath, exc_info=True)
@@ -342,14 +400,17 @@ class ProxyInstanceProcess(Process):
             if caps.boolget("challenge"):
                 self.send_disconnect(proto, AUTHENTICATION_ERROR, "this socket does not use authentication")
                 return
-            if caps.get("info_request", False):
+            generic_request = caps.strget("request")
+            def is_req(mode):
+                return generic_request==mode or caps.boolget("%s_request" % mode)
+            if is_req("info"):
                 proto.send_now(("hello", self.get_proxy_info(proto)))
                 self.timeout_add(5*1000, self.send_disconnect, proto, CLIENT_EXIT_TIMEOUT, "info sent")
                 return
-            elif caps.get("stop_request", False):
+            elif is_req("stop"):
                 self.stop("socket request", None)
                 return
-            elif caps.get("version_request", False):
+            elif is_req("version"):
                 version = XPRA_VERSION
                 if caps.boolget("full-version-request"):
                     version = full_version_str()
@@ -358,11 +419,11 @@ class ProxyInstanceProcess(Process):
                 return
         self.send_disconnect(proto, CONTROL_COMMAND_ERROR, "this socket only handles 'info', 'version' and 'stop' requests")
 
-    def send_disconnect(self, proto, reason, *extra):
-        log("send_disconnect(%s, %s, %s)", proto, reason, extra)
+    def send_disconnect(self, proto, *reasons):
+        log("send_disconnect(%s, %s)", proto, reasons)
         if proto._closed:
             return
-        proto.send_now(["disconnect", reason]+list(extra))
+        proto.send_disconnect(reasons)
         self.timeout_add(1000, self.force_disconnect, proto)
 
     def force_disconnect(self, proto):
@@ -414,6 +475,10 @@ class ProxyInstanceProcess(Process):
 
     def filter_client_caps(self, caps):
         fc = self.filter_caps(caps, ("cipher", "challenge", "digest", "aliases", "compression", "lz4", "lz0", "zlib"))
+        #the display string may override the username:
+        username = self.disp_desc.get("username")
+        if username:
+            fc["username"] = username
         #update with options provided via config if any:
         fc.update(self.sanitize_session_options(self.session_options))
         #add video proxies if any:
@@ -500,7 +565,7 @@ class ProxyInstanceProcess(Process):
         for proto in (self.client_protocol, self.server_protocol):
             if proto and proto!=skip_proto:
                 log("sending disconnect to %s", proto)
-                proto.flush_then_close(["disconnect", SERVER_SHUTDOWN, reason])
+                proto.send_disconnect([SERVER_SHUTDOWN, reason])
 
 
     def queue_client_packet(self, packet):
@@ -511,8 +576,9 @@ class ProxyInstanceProcess(Process):
     def get_client_packet(self):
         #server wants a packet
         p = self.client_packets.get()
-        log("sending to client: %s", p[0])
-        return p, None, None, self.client_packets.qsize()>0
+        s = self.client_packets.qsize()
+        log("sending to client: %s (queue size=%i)", p[0], s)
+        return p, None, None, None, True, s>0
 
     def process_client_packet(self, proto, packet):
         packet_type = packet[0]
@@ -545,15 +611,16 @@ class ProxyInstanceProcess(Process):
     def get_server_packet(self):
         #server wants a packet
         p = self.server_packets.get()
-        log("sending to server: %s", p[0])
-        return p, None, None, self.server_packets.qsize()>0
+        s = self.server_packets.qsize()
+        log("sending to server: %s (queue size=%i)", p[0], s)
+        return p, None, None, None, True, s>0
 
 
     def _packet_recompress(self, packet, index, name):
         if len(packet)>index:
             data = packet[index]
             if len(data)<512:
-                packet[index] = str(data)
+                packet[index] = strtobytes(data)
                 return
             #FIXME: this is ugly and not generic!
             zlib = compression.use_zlib and self.caps.boolget("zlib", True)
@@ -638,25 +705,39 @@ class ProxyInstanceProcess(Process):
             if packet[3]:
                 packet[3] = Compressed("file-chunk-data", packet[3])
         elif packet_type=="challenge":
-            from xpra.net.crypto import get_salt, get_digest_module
-            #client may have already responded to the challenge,
-            #so we have to handle authentication from this end
-            salt = packet[1]
-            digest = packet[3]
-            client_salt = get_salt(len(salt))
-            salt = xor_str(salt, client_salt)
-            digestmod = get_digest_module(digest)
-            if not digestmod:
-                self.stop("digest mode '%s' not supported", std(digest))
-                return
-            password = self.session_options.get("password")
+            password = self.disp_desc.get("password", self.session_options.get("password"))
+            log("password from %s / %s = %s", self.disp_desc, self.session_options, password)
             if not password:
                 self.stop("authentication requested by the server, but no password available for this session")
                 return
-            import hmac
-            password = strtobytes(password)
-            salt = memoryview_to_bytes(salt)
-            challenge_response = hmac.HMAC(password, salt, digestmod=digestmod).hexdigest()
+            from xpra.net.crypto import get_salt, gendigest
+            #client may have already responded to the challenge,
+            #so we have to handle authentication from this end
+            server_salt = bytestostr(packet[1])
+            l = len(server_salt)
+            digest = bytestostr(packet[3])
+            salt_digest = "xor"
+            if len(packet)>=5:
+                salt_digest = bytestostr(packet[4])
+            if salt_digest in ("xor", "des"):
+                if not LEGACY_SALT_DIGEST:
+                    self.stop("server uses legacy salt digest '%s'" % salt_digest)
+                    return
+                log.warn("Warning: server using legacy support for '%s' salt digest", salt_digest)
+            if salt_digest=="xor":
+                #with xor, we have to match the size
+                assert l>=16, "server salt is too short: only %i bytes, minimum is 16" % l
+                assert l<=256, "server salt is too long: %i bytes, maximum is 256" % l
+            else:
+                #other digest, 32 random bytes is enough:
+                l = 32
+            client_salt = get_salt(l)
+            salt = gendigest(salt_digest, client_salt, server_salt)
+            challenge_response = gendigest(digest, password, salt)
+            if not challenge_response:
+                log("invalid digest module '%s': %s", digest)
+                self.stop("server requested '%s' digest but it is not supported" % digest)
+                return
             log.info("sending %s challenge response", digest)
             self.send_hello(challenge_response, client_salt)
             return
@@ -793,8 +874,9 @@ class ProxyInstanceProcess(Process):
                 #no video encoder!
                 enc_pillow = get_codec("enc_pillow")
                 if not enc_pillow:
-                    from xpra.server.picture_encode import warn_encoding_once
-                    warn_encoding_once("no-video-no-PIL", "no video encoder found for rgb format %s, sending as plain RGB!" % rgb_format)
+                    if first_time("no-video-no-PIL-%s" % rgb_format):
+                        enclog.warn("Warning: no video encoder found for rgb format %s", rgb_format)
+                        enclog.warn(" sending as plain RGB")
                     return passthrough(True)
                 enclog("no video encoder available: sending as jpeg")
                 coding, compressed_data, client_options, _, _, _, _ = enc_pillow.encode("jpeg", image, quality, speed, False)
@@ -830,7 +912,7 @@ class ProxyInstanceProcess(Process):
         #so we just ask the encode thread (which deals with encoders already)
         #to do what may need to be done if we find a timeout:
         now = monotonic_time()
-        for wid in list(self.video_encoders_last_used_time.keys()):
+        for wid in tuple(self.video_encoders_last_used_time.keys()):
             idle_time = int(now-self.video_encoders_last_used_time.get(wid))
             if idle_time is None:
                 continue
