@@ -1,13 +1,14 @@
 # This file is part of Xpra.
-# Copyright (C) 2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2017-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import sys
 import os.path
 
-from xpra.os_util import OSX, POSIX, shellsub, getuid, getgid
-from xpra.platform.dotxpra import osexpand, norm_makepath
+from xpra.util import envbool
+from xpra.os_util import OSX, POSIX, shellsub, getuid, getgid, get_util_logger, osexpand
+from xpra.platform.dotxpra import norm_makepath
 from xpra.scripts.config import InitException
 
 
@@ -102,8 +103,7 @@ def write_runner_shell_scripts(contents, overwrite=True):
     # is running on the remote host.  Might need to revisit this later if
     # people run into problems or autodiscovery turns out to be less useful
     # than expected.
-    from xpra.log import Logger
-    log = Logger("server")
+    log = get_util_logger()
     from xpra.platform.paths import get_script_bin_dirs
     for d in get_script_bin_dirs():
         scriptdir = osexpand(d)
@@ -111,7 +111,8 @@ def write_runner_shell_scripts(contents, overwrite=True):
             try:
                 os.mkdir(scriptdir, 0o700)
             except Exception as e:
-                log.warn("Warning: failed to write script file in '%s':", scriptdir)
+                log("os.mkdir(%s, 0o700)", scriptdir, exc_info=True)
+                log.warn("Warning: failed to create script directory '%s':", scriptdir)
                 log.warn(" %s", e)
                 if scriptdir.startswith("/var/run/user") or scriptdir.startswith("/run/user"):
                     log.warn(" ($XDG_RUNTIME_DIR has not been created?)")
@@ -127,8 +128,9 @@ def write_runner_shell_scripts(contents, overwrite=True):
                 umask = os.umask(0)
                 os.umask(umask)
                 os.fchmod(scriptfile.fileno(), 0o700 & ~umask)
-                scriptfile.write(contents)
+                scriptfile.write(contents.encode())
         except Exception as e:
+            log("writing to %s", scriptpath, exc_info=True)
             log.error("Error: failed to write script file '%s':", scriptpath)
             log.error(" %s\n", e)
 
@@ -221,8 +223,7 @@ def daemonize():
 
 
 def write_pidfile(pidfile, uid, gid):
-    from xpra.log import Logger
-    log = Logger("server")
+    log = get_util_logger()
     pidstr = str(os.getpid())
     try:
         with open(pidfile, "w") as f:
@@ -257,3 +258,154 @@ def write_pidfile(pidfile, uid, gid):
     except Exception as e:
         log.error("Error: failed to write pid %i to pidfile '%s':", os.getpid(), pidfile)
         log.error(" %s", e)
+
+
+def get_uinput_device_path(device):
+    log = get_util_logger()
+    try:
+        log("get_uinput_device_path(%s)", device)
+        fd = device._Device__uinput_fd
+        log("fd(%s)=%s", device, fd)
+        import fcntl        #@UnresolvedImport
+        import ctypes
+        l = 16
+        buf = ctypes.create_string_buffer(l)
+        _IOC_READ = 2
+        #this magic value was calculated using the C macros:
+        l = fcntl.ioctl(fd, 2148554028, buf)
+        if l>0 and l<16:
+            virt_dev_path = buf.raw[:l].rstrip("\0")
+            log("UI_GET_SYSNAME(%s)=%s", fd, virt_dev_path)
+            uevent_path = b"/sys/devices/virtual/input/%s" % virt_dev_path
+            event_dirs = [x for x in os.listdir(uevent_path) if x.startswith("event")]
+            log("event dirs(%s)=%s", uevent_path, event_dirs)
+            for d in event_dirs:
+                uevent_filename = os.path.join(uevent_path, d, "uevent")
+                uevent_conf = open(uevent_filename, "rb").read()
+                for line in uevent_conf.splitlines():
+                    if line.find(b"=")>0:
+                        k,v = line.split(b"=", 1)
+                        log("%s=%s", k, v)
+                        if k==b"DEVNAME":
+                            dev_path = b"/dev/%s" % v
+                            log("found device path: %s" % dev_path)
+                            return dev_path
+    except Exception as e:
+        log.error("Error: cannot query uinput device path:")
+        log.error(" %", e)
+    return None
+
+def has_uinput():
+    try:
+        import uinput
+        assert uinput
+    except (ImportError, NameError) as e:
+        log = get_util_logger()
+        log("has_uinput()", exc_info=True)
+        log.info("cannot access python uinput module:")
+        log.info(" %s", e)
+        return False
+    try:
+        uinput.fdopen()
+    except Exception as e:
+        log = get_util_logger()
+        log("has_uinput()", exc_info=True)
+        log.info("cannot use uinput for virtual devices:")
+        log.info(" %s", e)
+        return False
+    return True
+
+def create_uinput_device(uuid, uid, events, name):
+    log = get_util_logger()
+    import uinput
+    BUS_USB = 0x03
+    #BUS_VIRTUAL = 0x06
+    VENDOR = 0xffff
+    PRODUCT = 0x1000
+    #our xpra_udev_product_version script will use the version attribute to set
+    #the udev OWNER value
+    VERSION = uid
+    try:
+        device = uinput.Device(events, name=name, bustype=BUS_USB, vendor=VENDOR, product=PRODUCT, version=VERSION)
+    except OSError as e:
+        log("uinput.Device creation failed", exc_info=True)
+        if os.getuid()==0:
+            #running as root, this should work!
+            log.error("Error: cannot open uinput,")
+            log.error(" make sure that the kernel module is loaded")
+            log.error(" and that the /dev/uinput device exists:")
+            log.error(" %s", e)
+        else:
+            log.info("cannot access uinput: %s", e)
+        return None
+    dev_path = get_uinput_device_path(device)
+    if not dev_path:
+        device.destroy()
+        return None
+    return name, device, dev_path
+
+def create_uinput_pointer_device(uuid, uid):
+    if not envbool("XPRA_UINPUT_POINTER", True):
+        return
+    import uinput
+    events = (
+        uinput.REL_X,
+        uinput.REL_Y,
+        uinput.REL_WHEEL,
+        #REL_HIRES_WHEEL = 0x10
+        #uinput.REL_HWHEEL,
+        uinput.BTN_LEFT,
+        uinput.BTN_RIGHT,
+        uinput.BTN_MIDDLE,
+        uinput.BTN_SIDE,
+        uinput.BTN_EXTRA,
+        uinput.BTN_FORWARD,
+        uinput.BTN_BACK,
+        )
+    name = "Xpra Virtual Pointer %s" % uuid
+    return create_uinput_device(uuid, uid, events, name)
+
+def create_uinput_touchpad_device(uuid, uid):
+    if not envbool("XPRA_UINPUT_TOUCHPAD", False):
+        return
+    import uinput
+    events = (
+        uinput.BTN_TOUCH,
+        uinput.ABS_X + (0, 2**24-1, 0, 0),
+        uinput.ABS_Y + (0, 2**24-1, 0, 0),
+        uinput.ABS_PRESSURE + (0, 255, 0, 0),
+        #uinput.BTN_TOOL_PEN,
+        )
+    name = "Xpra Virtual Touchpad %s" % uuid
+    return create_uinput_device(uuid, uid, events, name)
+
+
+def create_uinput_devices(uinput_uuid, uid):
+    log = get_util_logger()
+    try:
+        import uinput
+        assert uinput
+    except (ImportError, NameError) as e:
+        log.error("Error: cannot access python uinput module:")
+        log.error(" %s", e)
+        return {}
+    pointer = create_uinput_pointer_device(uinput_uuid, uid)
+    touchpad = create_uinput_touchpad_device(uinput_uuid, uid)
+    if not pointer and not touchpad:
+        return {}
+    def i(device):
+        if not device:
+            return {}
+        name, uinput_pointer, dev_path = device
+        return {
+            "name"      : name,
+            "uinput"    : uinput_pointer,
+            "device"    : dev_path,
+            }
+    return {
+        "pointer"   : i(pointer),
+        "touchpad"  : i(touchpad),
+        }
+
+def create_input_devices(uinput_uuid, uid):
+    return create_uinput_devices(uinput_uuid, uid)
